@@ -1,15 +1,14 @@
 #
-# $Id: UTF7.pm,v 2.9 2017/04/21 05:20:14 dankogai Exp dankogai $
+# $Id: UTF7.pm,v 2.10 2017/06/10 17:23:50 dankogai Exp dankogai $
 #
 package Encode::Unicode::UTF7;
 use strict;
 use warnings;
-no warnings 'redefine';
 use parent qw(Encode::Encoding);
 __PACKAGE__->Define('UTF-7');
-our $VERSION = do { my @r = ( q$Revision: 2.9 $ =~ /\d+/g ); sprintf "%d." . "%02d" x $#r, @r };
+our $VERSION = do { my @r = ( q$Revision: 2.10 $ =~ /\d+/g ); sprintf "%d." . "%02d" x $#r, @r };
 use MIME::Base64;
-use Encode;
+use Encode qw(find_encoding);
 
 #
 # Algorithms taken from Unicode::String by Gisle Aas
